<?php
/**
 * Phire CMS
 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.TXT.
 * It is also available through the world-wide-web at this URL:
 * http://www.phirecms.org/LICENSE.TXT
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@phirecms.org so we can send you a copy immediately.
 *
 * @category   Phire
 * @package    Phire_Parser
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 */

/**
 * Phire_Parser_Video
 *
 * @category   Phire
 * @package    Phire_Parser
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 * @version    1.1.2
 */

class Phire_Parser_Video
{

    /**
     * Video matches
     * @var array
     */
    protected $_matches = array('youtube' => array(),
                                'vimeo' => array(),
                                'viddler' => array());

    /**
     * Video RegEx
     * @var string
     */
    protected $_regexes = array('youtube' => '/\[\{youtube_(.*)\}\]/',
                                'vimeo' => '/\[\{vimeo_\w*\}\]/',
                                'viddler' => '/\[\{viddler_\w*\}\]/');

    /**
     * Content to evaluate
     * @var string
     */
    protected $_content = null;

    /**
     * URL request
     * @var string
     */
    protected $_url = null;

    /**
     * YouTube template
     * @var string
     */
    protected $_youtube_tmpl = "        <object width=\"425\" height=\"344\">\n            <param name=\"movie\" value=\"http://www.youtube.com/v/[{video_id}]\"></param>\n            <param name=\"allowFullScreen\" value=\"true\"></param>\n            <param name=\"wmode\" value=\"transparent\"></param>\n            <param name=\"allowscriptaccess\" value=\"always\"></param>\n            <embed src=\"http://www.youtube.com/v/[{video_id}]\" type=\"application/x-shockwave-flash\" allowscriptaccess=\"always\" wmode=\"transparent\" allowfullscreen=\"true\" width=\"425\" height=\"344\"></embed>\n        </object>";

    /**
     * Vimeo template
     * @var string
     */
    protected $_vimeo_tmpl = "        <object width=\"400\" height=\"300\">\n            <param name=\"allowfullscreen\" value=\"true\" />\n            <param name=\"allowscriptaccess\" value=\"always\" />\n            <param name=\"wmode\" value=\"transparent\" />\n            <param name=\"movie\" value=\"http://vimeo.com/moogaloop.swf?clip_id=[{video_id}]&amp;server=vimeo.com&amp;show_title=1&amp;show_byline=1&amp;show_portrait=0&amp;color=&amp;fullscreen=1\" />\n            <embed src=\"http://vimeo.com/moogaloop.swf?clip_id=[{video_id}]&amp;server=vimeo.com&amp;show_title=1&amp;show_byline=1&amp;show_portrait=0&amp;color=&amp;fullscreen=1\" type=\"application/x-shockwave-flash\" wmode=\"transparent\" allowfullscreen=\"true\" allowscriptaccess=\"always\" width=\"400\" height=\"300\"></embed>\n        </object>";

    /**
     * Vimeo template
     * @var string
     */
    protected $_viddler_tmpl = "        <object classid=\"clsid:D27CDB6E-AE6D-11cf-96B8-444553540000\" width=\"437\" height=\"370\" id=\"viddler_[{video_id}]\">\n            <param name=\"movie\" value=\"http://www.viddler.com/player/[{video_id}]/\" />\n            <param name=\"allowScriptAccess\" value=\"always\" />\n            <param name=\"wmode\" value=\"transparent\" />\n            <param name=\"allowFullScreen\" value=\"true\" />\n            <embed src=\"http://www.viddler.com/player/[{video_id}]/\" width=\"437\" height=\"370\" type=\"application/x-shockwave-flash\" allowScriptAccess=\"always\" wmode=\"transparent\" allowFullScreen=\"true\" name=\"viddler_[{video_id}]\"></embed>\n        </object>";


    /**
     * Constructor
     *
     * Instantiate the video parser object.
     *
     * @param  string $content
     * @param  string $url
     * @return void
     */
    public function __construct($content, $url = null)
    {

        // Perform a preg_match_all to look for any video tags within the content, then parse the content.
        $this->_content = $content;
        preg_match_all($this->_regexes['youtube'], $this->_content, $this->_matches['youtube']);
        preg_match_all($this->_regexes['vimeo'], $this->_content, $this->_matches['vimeo']);
        preg_match_all($this->_regexes['viddler'], $this->_content, $this->_matches['viddler']);
        $this->_parse();

    }

    /**
     * Static method to instantiate the video parser object
     * and return the parsed string.
     *
     * @param  string $cont
     * @param  string $u
     * @return Phire_Parser_Video
     */
    public static function init($cont, $u = null)
    {

        return new self($cont, $u);

    }

    /**
     * Set content within the content property.
     *
     * @param  string $subject
     * @param  string $replace
     * @return void
     */
    public function setContent($subject, $replace)
    {

        $this->_content = str_replace($subject, $replace, $this->_content);

    }

    /**
     * Get content within the content property.
     *
     * @return string
     */
    public function getContent()
    {

        return $this->_content;

    }

    /**
     * Renders any video elements within the content.
     *
     * @return void
     */
    protected function _parse()
    {

        // If YouTube video tag elements are found, format and set in content.
        if (isset($this->_matches['youtube'][0][0])) {

            foreach ($this->_matches['youtube'][0] as $value) {

                $video_format = '';

                $vid = str_replace('[{youtube_', '', $value);
                $vid = str_replace('}]', '', $vid);

                try {
                    $this->setContent($value, str_replace('[{video_id}]', $vid, $this->_youtube_tmpl));
                } catch (Exception $e) {
                    $this->setContent($value, '');
                }

            }

        }

        // If Vimeo video tag elements are found, format and set in content.
        if (isset($this->_matches['vimeo'][0][0])) {

            foreach ($this->_matches['vimeo'][0] as $value) {

                $video_format = '';

                $vid = str_replace('[{vimeo_', '', $value);
                $vid = str_replace('}]', '', $vid);

                try {
                    $this->setContent($value, str_replace('[{video_id}]', $vid, $this->_vimeo_tmpl));
                } catch (Exception $e) {
                    $this->setContent($value, '');
                }

            }

        }

        // If Viddler video tag elements are found, format and set in content.
        if (isset($this->_matches['viddler'][0][0])) {

            foreach ($this->_matches['viddler'][0] as $value) {

                $video_format = '';

                $vid = str_replace('[{viddler_', '', $value);
                $vid = str_replace('}]', '', $vid);

                try {
                    $this->setContent($value, str_replace('[{video_id}]', $vid, $this->_viddler_tmpl));
                } catch (Exception $e) {
                    $this->setContent($value, '');
                }

            }

        }

    }

}
